class Card:
    
    def __init__(self, number, suit, value):
        '''
        Class representing an individual playing card

        Attributes:
            number (string or int) - card name or number
            suit (string) - card suit: hearts, diamonds, clubs or spades
            value (int or list of [int, int] ) - card value, or if the card is an ace, list of card values (1 or 11)
        '''
        self.card_no = number
        self.suit = suit
        self.value = value
        
        
    def print_card(self):
        '''Function to print to screen the card number and suit
        '''
        print('{} of {}'.format(self.card_no, self.suit))



from .Card import Card

class Person:

    def __init__(self, name):
        '''
        Generic parent class representing a game participant

        Attributes:
            name (string) - participant name
            cards (set) - set of cards the participant has been dealt
        '''
        self.name = name
        self.cards = set()
        
        
    def print_cards(self):
        '''Function to print to screen all participant's cards and their summed value
        '''
        print('{}\'s Cards:'.format(self.name))
        for card in self.cards:
            card.print_card()
        print('{}\'s Cards Value: {}\n'.format(self.name, self.sum_cards()))
    
    
    def sum_cards(self):
        '''Function to sum participant's cards value
            Args: None
            Return:
                value (int or list of [int, int])
        '''
        #Set zero value as list so that both possible ace [1, 11] values can be added to it
        value = [0, 0]
        
        #Separate any aces from the non-ace cards
        aces = [x for x in self.cards if x.card_no == 'ace']
        non_ace_cards  = [x for x in self.cards if x.card_no != 'ace']
        
        #Only add both ace values for the 1st occurance of an ace in the participant's hand
        #Any more +11s will only cause the hand to bust so is not a real option
        x=1
        for card in aces:
            if x == 1:
                value[0] += card.value[0]
                value[1] += card.value[1]
                x += 1
            else:
                value[0] += card.value[0]
                value[1] += card.value[0]
        
        #For non-ace cards add card values to both value list items
        for card in non_ace_cards:
            value[0] += card.value
            value[1] += card.value
        
        #If both summed elements are the same then no ace has been involved and just return 1 value
        if value[0] == value[1]:
            return value[0]
        else:
            if value[1] > 21: #If the larger element is over 21 then it's bust so just return 1st element
                return value[0]
            else:
                return value
    
    
    def max_card_value(self):
        '''Function to return the maximum value of the participant's cards, if there is a choice of 2
            Args: None
            Return:
                value (int)
        '''
        if isinstance(self.sum_cards(), list):
            value = self.sum_cards()[1]
        else:
            value = self.sum_cards()
        return value
    

class Dealer(Person):

    def __init__(self, name):
        '''
        Child class inheriting from Person, representing the dealer of the game

        Attributes:
            name (string) - dealer name
            cards (set) - set of cards the dealer has been dealt
        '''
        Person.__init__(self, name)
        
    
class Player(Person):
    
    def __init__(self, name):
        '''
        Child class inheriting from Person, representing the game player

        Attributes:
            name (string) - player name
            cards (set) - set of cards the player has been dealt
            score (int) - player score
        '''
        Person.__init__(self, name)
        self.score = 0
        
    
    def bust(self):
        '''Function to return True if the player's cards total over 21
            Args: None
            Return:
                Boolean
        '''
        if self.max_card_value() > 21:
            return True
        else:
            return False




from .Person import Player, Dealer
from .Card import Card

class Game:
    
    def __init__(self, name):
        '''
        Class representing the game itself
        
        Args: name (string) - player name

        Attributes:
            deck (set of instances of Card) - set of cards, initially empty
            dealer (instance of Dealer) - create new dealer using 'Dealer' as name
            player (instance of Player) - create new player using args 'name'
        '''
        self.deck = set()
        self.dealer = Dealer('Dealer')
        self.player = Player(name)
        
        
    def new_deck(self):
        '''Function to update game.deck with a new deck of cards
            Args: None
            Return:
                new_deck (set of Cards)
        '''
        suits = {'hearts','diamonds','clubs','spades'}
        card_dict = {'ace': [1, 11], 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8, 9: 9,
                   'jack':10, 'queen':10, 'king':10} #card numbers and their values
        new_deck = set()
        for suit in suits:
            for no in card_dict:
                new_deck.add(Card(no, suit, card_dict[no]))
        return new_deck
        
        
    def deal_cards(self):
        '''Function to deal 2 cards each to player and dealer
        '''
        self.deck = self.new_deck() #Create new deck of cards
        self.player.cards = set()
        self.dealer.cards = set()
        for x in range(2):
            self.player.cards.add(self.deck.pop())
            self.dealer.cards.add(self.deck.pop())
        self.print_cards('deal') #Print dealt cards to screen and compare totals

        
    def hit(self):
        '''Function to deal the player 1 additional card, if has chosen to 'hit', then compare totals
        '''
        self.player.cards.add(self.deck.pop())
        if self.player.bust(): #If player has bust (cards total over 21) print results
            self.print_cards('hit')
        else:
            self.dealer_check() #Otherwise update dealer's cards
            self.print_cards('hit')

            
    def dealer_check(self):
        '''Function to add 1 card to the dealer's hand if current total < 17
        '''
        value = self.dealer.sum_cards()
        if isinstance(value, list):
            if value[1] < 17:
                self.dealer.cards.add(self.deck.pop())
        else:
            if value < 17:
                self.dealer.cards.add(self.deck.pop())
        
        
    def stick(self):
        '''If player chooses to 'stick', function to add to the dealer's cards until cards value = 17 or bust
        '''
        while self.dealer.max_card_value() < 17:
            self.dealer_check()
        self.print_cards('stick') #Print dealt cards to screen and compare totals
    
    
    def print_cards(self, hit_stick):
        '''Function to print players and dealer cards to screen and compare totals
        '''
        self.player.print_cards()
        self.dealer.print_cards()
        self.compare_card_values(hit_stick)
    
    def compare_card_values(self, hit_stick):
        '''Function to compare player and dealer card total values to determine player win or lose
        
            Args:
                hit_stick (string) - the name of the function calling for the comparison
                                    if is 'stick' then no further game play, so compare final totals
        '''
        p_value = self.player.max_card_value()
        d_value = self.dealer.max_card_value()
        
        #If player has 21 or dealer is bust then print win
        if (p_value == 21 and d_value != 21) or (p_value < 21 and d_value > 21):
            self.print_win_lose('win')
        
        #If player is bust or dealer has 21 then print lose
        elif (p_value > 21) or (p_value != 21 and d_value == 21) :
            self.print_win_lose('lose')

        #If player has chosen to stick then player wins if total is greater than dealer's
        elif hit_stick == 'stick':
            if p_value > d_value:
                self.print_win_lose('win')
            elif p_value == d_value:
                self.print_win_lose('draw')
            else:
                self.print_win_lose('lose')
                
                
    def print_win_lose(self, win_lose):
        '''Function to print win or lose results to screen and update player's score
        '''
        if win_lose == 'win':
            print('Congratulations! You win')
            self.player.score += 1
        elif win_lose == 'lose':
            print('Sorry. You lose')
            self.player.score -= 1
        else:
            print('Its a draw. Play again?')
        print('{}\'s Score: {}'.format(self.player.name, self.player.score))
        



from setuptools import setup

setup(name='blackjack_am',
      version='0.2',
      description='Blackjack (21) single player game',
      packages=['blackjack_am'],
      author='Alcia Moncrieffe',
      author_email='alcia.moncrieffe@gmail.com',
      zip_safe=False)
